// IxLinearDlg.cpp : implementation file
//

#include "stdafx.h"
#include "IxLinear.h"
#include "IxLinearDlg.h"
#include "..\Common\ceSDK.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CIxLinearDlg dialog

#define CMM_MAX_STR_LEN_AXIS_TITLE 50
#define IX_MAP_IDX	0  // Interpolation Map Index

// Interpolation Speed Type //
#define IX_MASTER_SPD_MODE	0 // Master Speed Mode
#define IX_VECTOR_SPD_MODE	1 // Vector Speed Mode

#define EXPECTED_NODECOUNT	10
#define TIMEOUT	(10*100) //  ϳ 100ms

long g_nErrCode;

void CIxLinearDlg::ShowCeSdkError(int nErrCode)
{
	CString sMsg;
	sMsg.Format("ceSDK error has been occured!\nerror code = %d", nErrCode);
	MessageBox(sMsg, "cnSDK Error", MB_OK|MB_ICONERROR);
}

CIxLinearDlg::CIxLinearDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CIxLinearDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CIxLinearDlg)
	m_bIsVectorSpeed = 0;
	m_fPosDataX = 10000.0;
	m_fPosDataY = 10000.0;
	m_bIsAbsPosMode = 0;
	m_nMastSpeedMode = 2;
	m_nVectSpeedMode = 2;
	m_fMastAcc = 100.0;
	m_fMastDec = 100.0;
	m_fMastVel = 100.0;
	m_fVectAcc = 100000.0;
	m_fVectDec = 100000.0;
	m_fVectVel = 10000.0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_bIsIxMapValid = FALSE;
}

void CIxLinearDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIxLinearDlg)
	DDX_Control(pDX, IDC_cmbAxisSelY, m_cmbAxisSelY);
	DDX_Control(pDX, IDC_cmbAxisSelX, m_cmbAxisSelX);
	DDX_Radio(pDX, IDC_rdoIxSpdType, m_bIsVectorSpeed);
	DDX_Text(pDX, IDC_edtPosX, m_fPosDataX);
	DDX_Text(pDX, IDC_edtPosY, m_fPosDataY);
	DDX_Radio(pDX, IDC_rdoPosType, m_bIsAbsPosMode);
	DDX_CBIndex(pDX, IDC_cmbMastSpeedMode, m_nMastSpeedMode);
	DDX_CBIndex(pDX, IDC_cmbVectSpeedMode, m_nVectSpeedMode);
	DDX_Text(pDX, IDC_edtMastAcc, m_fMastAcc);
	DDX_Text(pDX, IDC_edtMastDec, m_fMastDec);
	DDX_Text(pDX, IDC_edtMastVel, m_fMastVel);
	DDX_Text(pDX, IDC_edtVectAcc, m_fVectAcc);
	DDX_Text(pDX, IDC_edtVectDec, m_fVectDec);
	DDX_Text(pDX, IDC_edtVectVel, m_fVectVel);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CIxLinearDlg, CDialog)
	//{{AFX_MSG_MAP(CIxLinearDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_btnResetPos, OnResetPositions)
	ON_BN_CLICKED(IDC_btnGo, OnGo)
	ON_BN_CLICKED(IDC_btnStop, OnStop)
	ON_BN_CLICKED(IDC_rdoIxSpdType, OnIxSpdTypeChange)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_rdoIxSpdType2, OnIxSpdTypeChange)
	ON_BN_CLICKED(IDC_BTN_VIEWSTATEMON, OnBtnViewstatemon)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIxLinearDlg message handlers

BOOL CIxLinearDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// Load CMMSDK(DLL) Library //
	if(!ceLoadDll()){
		MessageBox("Can't load CE-SDK Library(DLL)", "Error", MB_OK | MB_ICONERROR);
		return TRUE;
	}

	// set debug mode enable
	//cnGnDebugMode(2, NULL);

	// Load Motion & DIO devices //
	long nNumNodes;
	int i;
		
	ceGnLoad();

	if((g_nErrCode = ceGnSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, false, &nNumNodes)) < 0) {
		ShowCeSdkError(g_nErrCode);
		return TRUE;
	}

	ceGnTotalMotionChannel( &m_nNumAxes );

	//ConfigMotion();
	m_pStateMonDlg = new CStateMonitorDlg(this);
	VERIFY(m_pStateMonDlg->Create(IDD_STATEMONITOR_DIALOG, this));

	////////////////////////////////////////////////////////////////////////////////////
	// ༳ ޺ڽ ฮƮ äֽϴ. 
	char szAxisTitle[CMM_MAX_STR_LEN_AXIS_TITLE];
	m_cmbAxisSelX.ResetContent(); // Clear combobox
	m_cmbAxisSelY.ResetContent(); // Clear combobox
	for(i=0; i<m_nNumAxes; i++){
		sprintf(szAxisTitle, "Axis %d", i);
		m_cmbAxisSelX.AddString(szAxisTitle);
		m_cmbAxisSelY.AddString(szAxisTitle);
	}
	m_cmbAxisSelX.SetCurSel(0);
	m_cmbAxisSelY.SetCurSel(1);

	OnIxSpdTypeChange(); 

	SetTimer(0, 100, NULL);
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}


void CIxLinearDlg::ConfigMotion()
{
	for(int i=0; i<m_nNumAxes; i++)
	{
		cemCfgMioProperty_Set(i, cemMPID_ALM_LOGIC, cemLOGIC_A);
		cemCfgMioProperty_Set(i, cemMPID_EL_LOGIC, cemLOGIC_A);
		cemCfgInMode_Set(i, cemIMODE_AB4X, false);
		cemCfgOutMode_Set(i, cemOMODE_CWCCW0);
	}
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CIxLinearDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CIxLinearDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CIxLinearDlg::OnTimer(UINT nIDEvent) 
{
	//-------  ġ ͸ -------------------------------//
	int nAxisX = m_cmbAxisSelX.GetCurSel();
	int nAxisY = m_cmbAxisSelY.GetCurSel();
	CString sPos;
	double fCurPos;

	// Command position (X) //
	cemStPosition_Get(nAxisX, cemCNT_COMM, &fCurPos);
	sPos.Format("%0.16G", fCurPos);
	GetDlgItem(IDC_lblPosC0)->SetWindowText(sPos);
	// Feedback position (X) //
	cemStPosition_Get(nAxisX, cemCNT_FEED, &fCurPos);
	sPos.Format("%0.16G", fCurPos);
	GetDlgItem(IDC_lblPosF0)->SetWindowText(sPos);
	// Command position (Y) //
	cemStPosition_Get(nAxisY, cemCNT_COMM, &fCurPos);
	sPos.Format("%0.16G", fCurPos);
	GetDlgItem(IDC_lblPosC1)->SetWindowText(sPos);
	// Feedback position (Y) //
	cemStPosition_Get(nAxisY, cemCNT_FEED, &fCurPos);
	sPos.Format("%0.16G", fCurPos);
	GetDlgItem(IDC_lblPosF1)->SetWindowText(sPos);
	
	CDialog::OnTimer(nIDEvent);
}

void CIxLinearDlg::OnResetPositions() 
{
	int nAxisX = m_cmbAxisSelX.GetCurSel();
	int nAxisY = m_cmbAxisSelY.GetCurSel();
	cemStPosition_Set(nAxisX, cemCNT_COMM, 0.0); // Reset command position of X-axis
	cemStPosition_Set(nAxisX, cemCNT_FEED, 0.0); // Reset feedback position of X-axis
	cemStPosition_Set(nAxisY, cemCNT_COMM, 0.0); // Reset command position of X-axis
	cemStPosition_Set(nAxisY, cemCNT_FEED, 0.0); // Reset feedback position of X-axis
}

void CIxLinearDlg::OnGo() 
{
	UpdateData();
	
	int nAxisX = m_cmbAxisSelX.GetCurSel();
	int nAxisY = m_cmbAxisSelY.GetCurSel();

	// X Y  ȣ̸  ó //
	if(nAxisX == nAxisY){
		MessageBox("Error: X and Y axes must be different from each other!", "Error", MB_OK | MB_ICONERROR);
		return;
	}
	
	// Interpolation   //
	long dwAxesMask1=0x0, dwAxesMask2=0x0;
	if(nAxisX < 32) dwAxesMask1 |= (1<<nAxisX); // X ȣ 0~31 쿡 Mask1 Ʈ ϰ...
	else dwAxesMask2 |= (1<<(nAxisX-32)); // X ȣ 32~63 쿡 Mask2 Ʈ Ѵ...
	if(nAxisY < 32) dwAxesMask1 |= (1<<nAxisY); // Y ȣ 0~31 쿡 Mask1 Ʈ ϰ...
	else dwAxesMask2 |= (1<<(nAxisY-32)); // Y ȣ 32~63 쿡 Mask2 Ʈ Ѵ...

	LONG nRet = 0;

	LONG nNodeIP, nNodeID, nNodeInGlobal, nModuleIdx, nModuleChnl;
	ceGnLocalAxis_Get( nAxisX, &nNodeIP, &nNodeID, &nNodeInGlobal, &nModuleIdx, &nModuleChnl );

	long nResult = cemIxMapAxes(IX_MAP_IDX, nNodeID, dwAxesMask1, dwAxesMask2);
	m_bIsIxMapValid = TRUE;
		
	// Speed Setting //
	if(m_bIsVectorSpeed){
		cemIxSpeedPattern_Set(IX_MAP_IDX, TRUE, m_nVectSpeedMode, m_fVectVel, m_fVectAcc, m_fVectDec);
	}else{
		cemIxSpeedPattern_Set(IX_MAP_IDX, FALSE, m_nMastSpeedMode, m_fMastVel, m_fMastAcc, m_fMastDec);
	}

	double fPosList[2] = {m_fPosDataX, m_fPosDataY};
	if(m_bIsAbsPosMode){
		if( (g_nErrCode = cemIxLineToStart(IX_MAP_IDX, fPosList)) != ceERR_NONE){
			//ShowCnSdkError(g_nErrCode);
		}
	}else{
		if((g_nErrCode = cemIxLineStart(IX_MAP_IDX, fPosList)) != ceERR_NONE){
			//ShowCnSdkError(g_nErrCode);
		}
	}
}

void CIxLinearDlg::OnStop() 
{
	//  ༳  ̷  쿡  ϹǷ ٷ Ѵ. //
	if(!m_bIsIxMapValid)	return;
	
	GetDlgItem(IDC_btnStop)->EnableWindow(FALSE);
	if((g_nErrCode=cemIxStop(IX_MAP_IDX)) != ceERR_NONE){
		ShowCeSdkError(g_nErrCode);
	}

	GetDlgItem(IDC_btnStop)->EnableWindow(TRUE);
}

void CIxLinearDlg::OnIxSpdTypeChange() 
{
	UpdateData();
	if(m_bIsVectorSpeed){
		GetDlgItem(IDC_cmbMastSpeedMode)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtMastVel)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtMastAcc)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtMastDec)->EnableWindow(FALSE);
		GetDlgItem(IDC_cmbVectSpeedMode)->EnableWindow(TRUE);
		GetDlgItem(IDC_edtVectVel)->EnableWindow(TRUE);
		GetDlgItem(IDC_edtVectAcc)->EnableWindow(TRUE);
		GetDlgItem(IDC_edtVectDec)->EnableWindow(TRUE);
	}else{
		GetDlgItem(IDC_cmbMastSpeedMode)->EnableWindow(TRUE);
		GetDlgItem(IDC_edtMastVel)->EnableWindow(TRUE);
		GetDlgItem(IDC_edtMastAcc)->EnableWindow(TRUE);
		GetDlgItem(IDC_edtMastDec)->EnableWindow(TRUE);
		GetDlgItem(IDC_cmbVectSpeedMode)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtVectVel)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtVectAcc)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtVectDec)->EnableWindow(FALSE);
	}	
}

void CIxLinearDlg::OnDestroy() 
{
	if(m_pStateMonDlg != NULL)
	{
		delete m_pStateMonDlg;
		m_pStateMonDlg = NULL;
	}
	
	ceGnUnload(); // Unload Device
	ceUnloadDll(); // Unload DLL 

	CDialog::OnDestroy();
}

void CIxLinearDlg::OnBtnViewstatemon() 
{
	if(m_pStateMonDlg)
	{
		m_pStateMonDlg->ShowWindow(SW_SHOW);
	}
}
