// OverridePositionDlg.cpp : implementation file
//

#include "stdafx.h"
#include "OverridePosition.h"
#include "OverridePositionDlg.h"
#include "SpeedSetup.h"
#include "..\Common\ceSDK.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define CMM_MAX_STR_LEN_AXIS_TITLE 50

#define EXPECTED_NODECOUNT	10
#define TIMEOUT	(10*100) //  ϳ 100ms

long g_nErrCode;

void COverridePositionDlg::ShowCeSdkError(int nErrCode)
{
	CString sMsg;
	sMsg.Format("ceSDK error has been occured!\nerror code = %d", nErrCode);
	MessageBox(sMsg, "ceSDK Error", MB_OK|MB_ICONERROR);
}

/////////////////////////////////////////////////////////////////////////////
// COverridePositionDlg dialog

COverridePositionDlg::COverridePositionDlg(CWnd* pParent /*=NULL*/)
	: CDialog(COverridePositionDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(COverridePositionDlg)
	m_fPosInput = 100000.0;
	m_nIsAbsMode = 1;
	m_nHardApply = 0;
	m_fOvrPosInput = 7000.0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_pStateMonDlg = NULL;
}

void COverridePositionDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COverridePositionDlg)
	DDX_Control(pDX, IDC_SLIDER1, m_sldSpeedRatio);
	DDX_Control(pDX, IDC_cmbAxisSel, m_cmbAxisSel);
	DDX_Text(pDX, IDC_edtPosVal, m_fPosInput);
	DDX_Text(pDX, IDC_EDT_OVRPOS, m_fOvrPosInput);
	DDX_Radio(pDX, IDC_rdoRelPos, m_nIsAbsMode);
	DDX_Check( pDX, IDC_CHK_HARDAPPLY, m_nHardApply );
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(COverridePositionDlg, CDialog)
	//{{AFX_MSG_MAP(COverridePositionDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_btnResetPos, OnResetPos)
	ON_BN_CLICKED(IDC_btnSpeedSetup, OnbtnSpeedSetup)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_btnMove, OnbtnMove)
	ON_BN_CLICKED(IDC_btnStop, OnbtnStop)
	ON_BN_CLICKED(IDC_BTN_VIEWSTATEMON, OnBtnViewstatemon)
	ON_BN_CLICKED( IDC_BTN_OVRGO, OnBtnOvrMove )
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COverridePositionDlg message handlers

BOOL COverridePositionDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// Load CMMSDK(DLL) Library //
	if(!ceLoadDll()){
		MessageBox("Can't load CE-SDK Library(DLL)", "Error", MB_OK | MB_ICONERROR);
		return TRUE;
	}

	// set debug mode enable
	//cnGnDebugMode(2, NULL);

	// Load Motion & DIO devices //
	long nNumNodes;
	int i;
		
	//̽ ε մϴ.
	g_nErrCode = ceGnLoad();

	if( g_nErrCode != ceERR_NONE ) 
	{
		ShowCeSdkError(g_nErrCode);
		return FALSE;
	}		

	long nIsSearchedDev = 0;

	//̹ ˻  Ȯմϴ.
	ceGnIsSearchedDevice( &nIsSearchedDev );

	if ( nIsSearchedDev == CE_FALSE )
	{
		//带 ˻մϴ.
		if((g_nErrCode = ceGnSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, FALSE, &nNumNodes)) < 0) {
			ShowCeSdkError(g_nErrCode);
			return FALSE;
		}		
	}
	else
	{
		//带 ˻մϴ.
		if((g_nErrCode = ceGnReSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, FALSE, &nNumNodes)) < 0) {
			ShowCeSdkError(g_nErrCode);
			return FALSE;
		}		
	}

	ceGnTotalMotionChannel( &m_nNumAxes );

	//ConfigMotion();
	m_pStateMonDlg = new CStateMonitorDlg(this);
	VERIFY(m_pStateMonDlg->Create(IDD_STATEMONITOR_DIALOG, this));

	////////////////////////////////////////////////////////////////////////////////////
	// ༳ ޺ڽ ฮƮ äֽϴ. 
	char szAxisTitle[CMM_MAX_STR_LEN_AXIS_TITLE];
	m_cmbAxisSel.ResetContent(); // Clear combobox
	for(i=0; i<m_nNumAxes; i++){
		sprintf(szAxisTitle, "Axis %d", i);
		m_cmbAxisSel.AddString(szAxisTitle);
	}
	m_cmbAxisSel.SetCurSel(0);
	// Speed ratio ̴    //
	m_sldSpeedRatio.SetRange(1, 100);
	m_sldSpeedRatio.SetPos(100);

	// Set timer to display current command position //
	SetTimer(0, 100, NULL); 
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}


void COverridePositionDlg::ConfigMotion()
{
	for(int i=0; i<m_nNumAxes; i++)
	{
		cemCfgMioProperty_Set(i, cemMPID_ALM_LOGIC, cemLOGIC_A);
		cemCfgMioProperty_Set(i, cemMPID_EL_LOGIC, cemLOGIC_A);
		cemCfgInMode_Set(i, cemIMODE_AB4X, false);
		cemCfgOutMode_Set(i, cemOMODE_CWCCW0);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void COverridePositionDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR COverridePositionDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


/***********************************************************************
* OnResetPos() : Position "Reset" ư ݹԼ. Command & Feedback 
* position 0 Ѵ.
***********************************************************************/
void COverridePositionDlg::OnResetPos() 
{
	int nAxis = m_cmbAxisSel.GetCurSel();
	cemStPosition_Set(nAxis, cemCNT_COMM, 0.0); // Reset COMMAND Position
	cemStPosition_Set(nAxis, cemCNT_FEED, 0.0); // Reset FEEDBACK Position
}

/***********************************************************************
* OnSpeedSetup() : "Speed Setup" Button's event handler. ӵ 
*    ִ ȭڸ ÷ϰ  ӵ Ʈѷ
* մϴ.   cmmCfgSetSpeedPattern() Լ  ϱ
*   ڵԴϴ. ׷ Ϲ ӵ CME Ͽ 
* ϹǷ cmmCfgSetSpeedPattern() Լ  ʿ䰡  찡 ϴ. 
* ſ, cmmSxSetSpeedRatio() Լ ̿Ͽ ̼ۼӵ ϴ 
* ϹԴϴ.
***********************************************************************/
void COverridePositionDlg::OnbtnSpeedSetup() 
{
	UpdateData(); 
	int nAxis = m_cmbAxisSel.GetCurSel();
	CSpeedSetup SpeedSetup;	
	SpeedSetup.SetAxis(nAxis);
	// ȭ ÷ //
	if(SpeedSetup.DoModal() == IDOK){
		// ڰ  ӵ  Ѵ. //
		cemSxOptIniSpeed_Set(nAxis, SpeedSetup.m_fVini);
		cemCfgSpeedPattern_Set(nAxis, SpeedSetup.m_nSpeedMode, SpeedSetup.m_fVwork,
			SpeedSetup.m_fAcc, SpeedSetup.m_fDec);
	}
}

/***********************************************************************
* OnbtnMove(): "Move/Override" ư ̺Ʈ ڵ鷯.   ִ 
* 쿡 "Start Move"  , ̼ 쿡 ̵ 
*  .
***********************************************************************/
void COverridePositionDlg::OnbtnMove() 
{
	UpdateData();

	int nAxis = m_cmbAxisSel.GetCurSel();

	double fVelR = (double)m_sldSpeedRatio.GetPos();
	// Speed ratio :  ǻ 100% 
	cemSxSpeedRatio_Set(nAxis, cemSMODE_KEEP/*Keep prev. setting*/, fVelR, 100, 100);
	
	if(m_nIsAbsMode)
	{
		cemSxMoveToStart(nAxis, m_fPosInput);
	}
	else
	{
		cemSxMoveStart(nAxis, m_fPosInput);
	}
}

/***********************************************************************
* OnbtnStop() : Position "Reset" ư ݹԼ. Command & Feedback 
* position 0 Ѵ.
***********************************************************************/
void COverridePositionDlg::OnbtnStop() 
{
	int nAxis = m_cmbAxisSel.GetCurSel();
	LONG nRetVal = 0;
	nRetVal = cemSxStop(nAxis, TRUE, FALSE);

}


/***********************************************************************
* OnTimer() : WM_TIMER event handler
* : Read current command position and display it
***********************************************************************/
void COverridePositionDlg::OnTimer(UINT nIDEvent) 
{
	UpdateData();

	//-------  ġ ͸ -------------------------------//
	int nAxis = m_cmbAxisSel.GetCurSel();
	CString sPos;
	double fCurPos;

	// Command position //
	cemStPosition_Get(nAxis, cemCNT_COMM, &fCurPos);
	sPos.Format("%0.16G", fCurPos);
	GetDlgItem(IDC_lblPosC)->SetWindowText(sPos);
	
	// Feedback position //
	cemStPosition_Get(nAxis, cemCNT_FEED, &fCurPos);
	sPos.Format("%0.16G", fCurPos);
	GetDlgItem(IDC_lblPosF)->SetWindowText(sPos);

	//-------- Speed ratio  ȭ鿡 ڷ ǥϱ ------------//
	int nRatio = m_sldSpeedRatio.GetPos();
	CString sTemp;

	sTemp.Format("%d (%s)", nRatio, "%");
	GetDlgItem(IDC_lblSpeedRatio)->SetWindowText(sTemp);

	CDialog::OnTimer(nIDEvent);
}

void COverridePositionDlg::OnDestroy() 
{
	CDialog::OnDestroy();

	if(m_pStateMonDlg)
	{
		delete m_pStateMonDlg;
		m_pStateMonDlg = NULL;
	}
	
	// TODO: Add your message handler code here
	int nAxis = m_cmbAxisSel.GetCurSel();
	cemSxStop(nAxis, FALSE, FALSE);
	ceGnUnload(); // Unload Device
	ceUnloadDll(); // Unload DLL 
}

void COverridePositionDlg::OnBtnViewstatemon() 
{
	if(m_pStateMonDlg)
		m_pStateMonDlg->ShowWindow(SW_SHOW);
	
}

void COverridePositionDlg::OnBtnOvrMove()
{
	UpdateData();
	
	int nAxis = m_cmbAxisSel.GetCurSel();	

	//   .
	// ӵ  //
	double fVelR = (double)m_sldSpeedRatio.GetPos();
	// Speed ratio :  ǻ 100% 
	cemSxSpeedRatio_Set(nAxis, cemSMODE_KEEP/*Keep prev. setting*/, fVelR, 100, 100);

	cemOverrideSpeedSet( nAxis );
	
	if(m_nIsAbsMode){
		
		long dwIsIgnored;
		
		// ̵  ġ ̵ մϴ.
		cemOverrideMoveTo(nAxis, m_fOvrPosInput, m_nHardApply, &dwIsIgnored);
		
	}else{
		long dwIsIgnored;
		
		// ̵  Ÿ ̵ մϴ.
		cemOverrideMove(nAxis, m_fOvrPosInput, m_nHardApply, &dwIsIgnored);
	}
}
