// OverrideSpeedDlg.cpp : implementation file
//

#include "stdafx.h"
#include "OverrideSpeed.h"
#include "OverrideSpeedDlg.h"
#include "..\Common\ceSDK.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define CMM_MAX_STR_LEN_AXIS_TITLE 50

#define EXPECTED_NODECOUNT	10
#define TIMEOUT	(10*100) //  ϳ 100ms

long g_nErrCode;

void COverrideSpeedDlg::ShowCeSdkError(int nErrCode)
{
	CString sMsg;
	sMsg.Format("ceSDK error has been occured!\nerror code = %d", nErrCode);
	MessageBox(sMsg, "cnSDK Error", MB_OK|MB_ICONERROR);
}

/////////////////////////////////////////////////////////////////////////////
// COverrideSpeedDlg dialog

COverrideSpeedDlg::COverrideSpeedDlg(CWnd* pParent /*=NULL*/)
	: CDialog(COverrideSpeedDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(COverrideSpeedDlg)
	m_fAccel = 0.0;
	m_fDecel = 0.0;
	m_fVel = 0.0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void COverrideSpeedDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COverrideSpeedDlg)
	DDX_Control(pDX, IDC_sldVelRatio, m_sldVelRatio);
	DDX_Control(pDX, IDC_sldDecRatio, m_sldDecRatio);
	DDX_Control(pDX, IDC_sldAccRatio, m_sldAccRatio);
	DDX_Control(pDX, IDC_cmbSpeedMode, m_cmbSpeedMode);
	DDX_Control(pDX, IDC_cmbAxisSel2, m_cmbAxisSel);
	DDX_Text(pDX, IDC_edtAcc, m_fAccel);
	DDX_Text(pDX, IDC_edtDec, m_fDecel);
	DDX_Text(pDX, IDC_edtVwork, m_fVel);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(COverrideSpeedDlg, CDialog)
	//{{AFX_MSG_MAP(COverrideSpeedDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_btnStart, OnStart)
	ON_BN_CLICKED(IDC_btnStop, OnStop)
	ON_CBN_SELCHANGE(IDC_cmbAxisSel, OnSelchangeAxisSel)
	ON_BN_CLICKED(IDC_btnOverPattern, OnOverrideSpeedPattern)
	ON_BN_CLICKED(IDC_btnOverRatio, OnOverrideSpeedRatio)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_BTN_VIEWSTATEMON, OnBtnViewstatemon)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COverrideSpeedDlg message handlers

BOOL COverrideSpeedDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// Load CMMSDK(DLL) Library //
	if(!ceLoadDll()){
		MessageBox("Can't load CE-SDK Library(DLL)", "Error", MB_OK | MB_ICONERROR);
		return TRUE;
	}

	// set debug mode enable
	//cnGnDebugMode(2, NULL);

	// Load Motion & DIO devices //
	long nNumNodes;
	int i;
		
	//̽ ε մϴ.
	g_nErrCode = ceGnLoad();

	if( g_nErrCode != ceERR_NONE ) 
	{
		ShowCeSdkError(g_nErrCode);
		return FALSE;
	}		

	long nIsSearchedDev = 0;

	//̹ ˻  Ȯմϴ.
	ceGnIsSearchedDevice( &nIsSearchedDev );

	if ( nIsSearchedDev == CE_FALSE )
	{
		//带 ˻մϴ.
		if((g_nErrCode = ceGnSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, FALSE, &nNumNodes)) < 0) {
			ShowCeSdkError(g_nErrCode);
			return FALSE;
		}		
	}
	else
	{
		//带 ˻մϴ.
		if((g_nErrCode = ceGnReSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, FALSE, &nNumNodes)) < 0) {
			ShowCeSdkError(g_nErrCode);
			return FALSE;
		}		
	}

	ceGnTotalMotionChannel( &m_nNumAxes );

	//ConfigMotion();
	m_pStateMonDlg = new CStateMonitorDlg(this);
	VERIFY(m_pStateMonDlg->Create(IDD_STATEMONITOR_DIALOG, this));

	////////////////////////////////////////////////////////////////////////////////////
	// ༳ ޺ڽ ฮƮ äֽϴ. 
	char szAxisTitle[CMM_MAX_STR_LEN_AXIS_TITLE];
	m_cmbAxisSel.ResetContent(); // Clear combobox
	for(i=0; i<m_nNumAxes; i++){
		sprintf(szAxisTitle, "Axis %d", i);
		m_cmbAxisSel.AddString(szAxisTitle);
	}
	m_cmbAxisSel.SetCurSel(0);
	m_cmbSpeedMode.SetCurSel(2);

	// Speed ratio ̴    //
	m_sldVelRatio.SetRange(1, 100);
	m_sldVelRatio.SetPos(100);
	m_sldAccRatio.SetRange(1, 100);
	m_sldAccRatio.SetPos(100);
	m_sldDecRatio.SetRange(1, 100);
	m_sldDecRatio.SetPos(100);
	
	SetTimer(0, 100, NULL); 
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void COverrideSpeedDlg::ConfigMotion()
{
	for(int i=0; i<m_nNumAxes; i++)
	{
		cemCfgMioProperty_Set(i, cemMPID_ALM_LOGIC, cemLOGIC_A);
		cemCfgMioProperty_Set(i, cemMPID_EL_LOGIC, cemLOGIC_A);
		cemCfgInMode_Set(i, cemIMODE_AB4X, false);
		cemCfgOutMode_Set(i, cemOMODE_CWCCW0);
	}
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void COverrideSpeedDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR COverrideSpeedDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


//**************************************************************************************
// OnStart(): "Start" ư ̺Ʈ ڵ鷯. (+)  ӵ  ̼ Ѵ.
// Ϲ ӵ  Ҷ ȯ漳 ݼӵ س α׷ 
// ӵ Ͽ ϴ  Ϲ̴.  Ϲδ cmmSxSetSpeedRatio()
//  ϸ ȴ.   cmmCfgSetSpeedPattern() ؼ Ǵ ݼӵ
// Overriding    ϱ ؼ cmmCfgSetSpeedPattern() Լ Բ Ͽ.
//**************************************************************************************
void COverrideSpeedDlg::OnStart() 
{
	UpdateData();
	int nAxis = m_cmbAxisSel.GetCurSel();
	int nSpeedMode = m_cmbSpeedMode.GetCurSel();

	// ݼӵ  //
	cemCfgSpeedPattern_Set(nAxis, nSpeedMode, m_fVel, m_fAccel, m_fDecel);
	// ӵ  //
	double fVelRatio = m_sldVelRatio.GetPos();
	double fAccRatio = m_sldAccRatio.GetPos();
	double fDecRatio = m_sldDecRatio.GetPos();
	cemSxSpeedRatio_Set(nAxis, cemSMODE_KEEP, fVelRatio, fAccRatio, fDecRatio);

	// մϴ.
	if((g_nErrCode = cemSxVMoveStart(nAxis, cemDIR_P)) != ceERR_NONE){ 
		ShowCeSdkError(g_nErrCode);
	}
}

void COverrideSpeedDlg::OnStop() 
{
	int nAxis = m_cmbAxisSel.GetCurSel();
	cemSxStop(nAxis, FALSE, TRUE);	
}


//**********************************************************************************
// OnSelchangeAxisSel(): "Axis" Combobox  Ǿ  ȣǴ ̺Ʈ ڵ鷯
//   ٲ ش   ӵ  о ȭ Ʈش.
//**********************************************************************************
void COverrideSpeedDlg::OnSelchangeAxisSel() 
{
	int nAxis = m_cmbAxisSel.GetCurSel();
	int nMode;
	cemCfgSpeedPattern_Get(nAxis, (long *)&nMode, &m_fVel, &m_fAccel, &m_fDecel);
	m_cmbSpeedMode.SetCurSel(nMode);

	//   .
	/*
	double fVelR, fAccR, fDecR;
	cemSxSpeedRatio_Get(nAxis, NULL, &fVelR, &fAccR, &fDecR);
	m_sldVelRatio.SetPos(fVelR);
	m_sldAccRatio.SetPos(fAccR);
	m_sldDecRatio.SetPos(fDecR);
	*/

	UpdateData(FALSE);
}


//**************************************************************************************
// OnTimer(): Timer ̺Ʈ ڵ鷯. 
//**************************************************************************************
void COverrideSpeedDlg::OnTimer(UINT nIDEvent) 
{
	int nAxis = m_cmbAxisSel.GetCurSel();

	CString sTemp;
	double fTemp;

	// Command position display //
	cemStPosition_Get(nAxis, cemCNT_COMM, &fTemp);
	sTemp.Format("%0.16G", fTemp);
	GetDlgItem(IDC_lblPosC)->SetWindowText(sTemp);

	// Feedback position display //
	cemStPosition_Get(nAxis, cemCNT_FEED, &fTemp);
	sTemp.Format("%0.16G", fTemp);
	GetDlgItem(IDC_lblPosF)->SetWindowText(sTemp);

	// Command velocity display //
	cemStSpeed_Get(nAxis, cemCNT_COMM, &fTemp);
	sTemp.Format("%0.1f", fTemp);
	GetDlgItem(IDC_lblVelC)->SetWindowText(sTemp);

	// Feedback velocity display //
	cemStSpeed_Get(nAxis, cemCNT_FEED, &fTemp);
	sTemp.Format("%0.1f", fTemp);
	GetDlgItem(IDC_lblVelF)->SetWindowText(sTemp);

	//-------- Speed/Acc/Dec ratio  ȭ鿡 ڷ ǥϱ ------------//
	INT nRatio;
	for(int i=0; i<3; i++){
		nRatio = ((CSliderCtrl*)GetDlgItem(IDC_sldVelRatio+i))->GetPos();
		sTemp.Format("%d (%s)", nRatio, "%");
		GetDlgItem(IDC_lblSpeedRatio+i)->SetWindowText(sTemp);
	}

	CDialog::OnTimer(nIDEvent);
}

//**************************************************************************************
// OnOverrideSpeedPattern(): "Speed Pattern" ׷ "Override" ư ̺Ʈ ڵ鷯.
// ݼӵ  ̵Ѵ. 
//**************************************************************************************
void COverrideSpeedDlg::OnOverrideSpeedPattern() 
{
	UpdateData();
	int nAxis = m_cmbAxisSel.GetCurSel();
	int nSpeedMode = m_cmbSpeedMode.GetCurSel();
	cemCfgSpeedPattern_Set(nAxis, nSpeedMode, m_fVel, m_fAccel, m_fDecel); // ̵ ӵ Ѵ.

	cemOverrideSpeedSet(nAxis); // ⿡  ӵ ̵ ȴ. 
}

//**************************************************************************************
// OnOverrideSpeedRatio(): "Speed Ratio" ׷ "Override" ư ̺Ʈ ڵ鷯.
// ӵ  ̵Ѵ. 
//**************************************************************************************
void COverrideSpeedDlg::OnOverrideSpeedRatio() 
{
	UpdateData();
	int nAxis = m_cmbAxisSel.GetCurSel();
	int nSpeedMode = m_cmbSpeedMode.GetCurSel();

	double fVelR = m_sldVelRatio.GetPos();
	double fAccR = m_sldAccRatio.GetPos();
	double fDecR = m_sldDecRatio.GetPos();
	cemSxSpeedRatio_Set(nAxis, nSpeedMode, fVelR, fAccR, fDecR); // ̵ ӵ Ѵ.

	cemOverrideSpeedSet(nAxis); // ⿡  ӵ ̵ ȴ. 
}

void COverrideSpeedDlg::OnDestroy() 
{
	CDialog::OnDestroy();

	if(m_pStateMonDlg)
	{
		delete m_pStateMonDlg;
		m_pStateMonDlg = NULL;
	}
	
	int nAxis = m_cmbAxisSel.GetCurSel();
	cemSxStop(nAxis, FALSE, TRUE);

	ceGnUnload();
	ceUnloadDll();
}

void COverrideSpeedDlg::OnBtnViewstatemon() 
{
	if(m_pStateMonDlg)
	{
		m_pStateMonDlg->ShowWindow(SW_SHOW);
	}
}
