// StateMonitorDlg.cpp : implementation file
//

#include "stdafx.h"
#include "StateMonitor.h"
#include "StateMonitorDlg.h"
#include "..\Common\ceSDK.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

enum _TLedID{LED_ALM, LED_ELN, LED_ELP, LED_ORG, LED_INP, LED_RDY, LED_DRP, LED_DRN, 
LED_CLR, LED_LTC, LED_SD, LED_STA, LED_STP, LED_ERC};

#define CMM_MAX_STR_LEN_AXIS_TITLE 50

#define EXPECTED_NODECOUNT	10
#define TIMEOUT	(10*100) //  ϳ 100ms

#define OFF 0

long g_nErrCode;

void CStateMonitorDlg::ShowCeSdkError(int nErrCode)
{
	CString sMsg;
	sMsg.Format("ceSDK error has been occured!\nerror code = %d", nErrCode);
	MessageBox(sMsg, "cnSDK Error", MB_OK|MB_ICONERROR);
}

/////////////////////////////////////////////////////////////////////////////
// CStateMonitorDlg dialog

CStateMonitorDlg::CStateMonitorDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CStateMonitorDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CStateMonitorDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CStateMonitorDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CStateMonitorDlg)
	DDX_Control(pDX, IDC_cmbAxisSel, m_cmbAxisSel);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CStateMonitorDlg, CDialog)
	//{{AFX_MSG_MAP(CStateMonitorDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_chkServoOn, OnchkServoOn)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_btnResetPos, OnbtnResetPos)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStateMonitorDlg message handlers

BOOL CStateMonitorDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	m_LedBmp[0].LoadBitmap(IDB_LedOff);
	m_LedBmp[1].LoadBitmap(IDB_LedBlue);
	m_LedBmp[2].LoadBitmap(IDB_LedRed);
	
	// Load CESDK(DLL) Library //
	if(!ceLoadDll()){
		MessageBox("Can't load CE-SDK Library(DLL)", "Error", MB_OK | MB_ICONERROR);
		return TRUE;
	}

	// set debug mode enable
	//ceGnDebugMode(2, NULL);

	// Load Motion & DIO devices //
	long nNumNodes = 0;
	int i;
		
	//̽ ε մϴ.
	g_nErrCode = ceGnLoad();

	if( g_nErrCode != ceERR_NONE ) 
	{
		ShowCeSdkError(g_nErrCode);
		return FALSE;
	}		

	long nIsSearchedDev = 0;

	//̹ ˻  Ȯմϴ.
	ceGnIsSearchedDevice( &nIsSearchedDev );

	if ( nIsSearchedDev == CE_FALSE )
	{
		//带 ˻մϴ.
		if((g_nErrCode = ceGnSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, FALSE, &nNumNodes)) < 0) {
			ShowCeSdkError(g_nErrCode);
			return FALSE;
		}		
	}
	else
	{
		//带 ˻մϴ.
		if((g_nErrCode = ceGnReSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, FALSE, &nNumNodes)) < 0) {
			ShowCeSdkError(g_nErrCode);
			return FALSE;
		}		
	}

	ceGnTotalMotionChannel( &m_nNumAxes );

	//ConfigMotion();

	////////////////////////////////////////////////////////////////////////////////////
	// ༳ ޺ڽ ฮƮ äֽϴ. 
	char szAxisTitle[CMM_MAX_STR_LEN_AXIS_TITLE];
	m_cmbAxisSel.ResetContent(); // Clear combobox
	for(i=0; i<m_nNumAxes; i++){
		sprintf(szAxisTitle, "Axis %d", i);
		m_cmbAxisSel.AddString(szAxisTitle);
	}
	m_cmbAxisSel.SetCurSel(0);
	
	// Set timer to display current command position //
	SetTimer(0, 100, NULL); 
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CStateMonitorDlg::ConfigMotion()
{
	for(int i=0; i<m_nNumAxes; i++)
	{
		cemCfgMioProperty_Set(i, cemMPID_ALM_LOGIC, cemLOGIC_A);
		cemCfgMioProperty_Set(i, cemMPID_EL_LOGIC, cemLOGIC_A);
		cemCfgInMode_Set(i, cemIMODE_AB4X, false);
		cemCfgOutMode_Set(i, cemOMODE_CWCCW0);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CStateMonitorDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CStateMonitorDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

//***************************************************************************************
// OnchkServoOn() : "Servo On" CheckBox ̺Ʈ ڵ鷯. Sevo ON/OFF Ѵ. 
//***************************************************************************************
void CStateMonitorDlg::OnchkServoOn() 
{
	int nAxis = m_cmbAxisSel.GetCurSel();
	int nCheck = ((CButton *)GetDlgItem(IDC_chkServoOn))->GetCheck();
	cemGnServoOn_Set(nAxis, nCheck);
}

CString CStateMonitorDlg::GetMstString(DWORD dwMst)
{
	switch (dwMst)
	{
	case cemMST_STOP:
		return "Stop";
	case cemMST_WAIT_DR:
		return "Wait DR";
	case cemMST_WAIT_STA:
		return "Wait STA";
	case cemMST_WAIT_INSYNC:
		return "Wait INSYNC";
	case cemMST_WAIT_OTHER:
		return "Wait OTHER";
	case cemMST_WAIT_ERC:
		return "Wait ERC";
	case cemMST_WAIT_DIR:
		return "Wait DIR";
	case cemMST_WAIT_PLSR:
		return "Wait PLSR";
	case cemMST_IN_RVSSPD:
		return "Wait RVSSPD";
	case cemMST_IN_INISPD:
		return "IniSpeed";
	case cemMST_IN_ACC:
		return "In Acceleration";
	case cemMST_IN_WORKSPD:
		return "In Workspeed";
	case cemMST_IN_DEC:
		return "In Deceleration";
	case cemMST_WAIT_INP:
		return "Wait INP";
	}

	CString sTemp;
	sTemp.Format("%d", dwMst);

	return sTemp;
}

void CStateMonitorDlg::OnTimer(UINT nIDEvent) 
{
	long dwTemp = 0;
	double fTemp;
	CString sTemp;
	int nAxis = m_cmbAxisSel.GetCurSel();
	
	// Motion State Display //
	CString szMotState;
	cemStReadMotionState(nAxis, &dwTemp); // read motion state code
	szMotState = GetMstString(dwTemp); // convert motion state code to string
	GetDlgItem(IDC_lblMst)->SetWindowText(szMotState); // display motion state string

	// Command position display //
	cemStPosition_Get(nAxis, cemCNT_COMM, &fTemp);
	sTemp.Format("%0.16G", fTemp);
	GetDlgItem(IDC_lblPosC)->SetWindowText(sTemp);

	// Feedback position display //
	cemStPosition_Get(nAxis, cemCNT_FEED, &fTemp);
	sTemp.Format("%0.16G", fTemp);
	GetDlgItem(IDC_lblPosF)->SetWindowText(sTemp);

	// Command velocity display //
	cemStSpeed_Get(nAxis, cemCNT_COMM, &fTemp);
	sTemp.Format("%0.1f", fTemp);
	GetDlgItem(IDC_lblVelC)->SetWindowText(sTemp);

	// Feedback velocity display //
	cemStSpeed_Get(nAxis, cemCNT_FEED, &fTemp);
	sTemp.Format("%0.1f", fTemp);
	GetDlgItem(IDC_lblVelF)->SetWindowText(sTemp);

	//---- MIO Display ----------------------------------------------------//
	cemStReadMioStatuses(nAxis, &dwTemp); // Read all MIO states
	SetLedState(LED_ALM, (dwTemp>>cemIOST_ALM) & 0x1); // ALM 
	SetLedState(LED_ELN, (dwTemp>>cemIOST_ELN) & 0x1); // +EL 
	SetLedState(LED_ELP, (dwTemp>>cemIOST_ELP) & 0x1); // -EL
	SetLedState(LED_ORG, (dwTemp>>cemIOST_ORG) & 0x1); // ORG
	SetLedState(LED_INP, (dwTemp>>cemIOST_INP) & 0x1); // INP
	SetLedState(LED_RDY, (dwTemp>>cemIOST_RDY) & 0x1); // RDY 
	SetLedState(LED_LTC, (dwTemp>>cemIOST_LTC) & 0x1); // LTC 
	SetLedState(LED_SD, (dwTemp>>cemIOST_SD) & 0x1); // SD 
	SetLedState(LED_STA, (dwTemp>>cemIOST_STA) & 0x1); // STA
	SetLedState(LED_STP, (dwTemp>>cemIOST_STP) & 0x1); // STP

	// Servo ON/OFF  ÷ //
	//cemGnServoOn_Get(nAxis, &dwTemp);	// FIXME: cemGnServoOn_Get Լ  ReadMioStatuses Լε on/off  Ȯ .
	CheckDlgButton(IDC_chkServoOn, (dwTemp>>cemIOST_SVON) & 0x1);

	CDialog::OnTimer(nIDEvent);
}


void CStateMonitorDlg::SetLedState(int nLedId, int nLedState)
{
	int nCntrID = IDC_LedALM + nLedId;

	if(nLedState == OFF){
		((CStatic *)GetDlgItem(nCntrID))->SetBitmap(m_LedBmp[0]); // OFF LED
	}else{
		switch(nLedId)
		{
		case LED_ALM:
		case LED_ELN:
		case LED_ELP:
			((CStatic *)GetDlgItem(nCntrID))->SetBitmap(m_LedBmp[2]); // Red LED
			break;

		default:
			((CStatic *)GetDlgItem(nCntrID))->SetBitmap(m_LedBmp[1]); // Blue LED
			break;
		}
	}
	
}

void CStateMonitorDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	
	KillTimer(0);
	ceGnUnload(); // Unload Device
	ceUnloadDll(); // Unload DLL 
}

void CStateMonitorDlg::OnbtnResetPos() 
{
	m_nAxis = m_cmbAxisSel.GetCurSel();
	cemStPosition_Set(m_nAxis, cemCNT_COMM, 0.0); // Reset COMMAND Position
	cemStPosition_Set(m_nAxis, cemCNT_FEED, 0.0); // Reset FEEDBACK Position	
}
