// SxMotionDlg.cpp : implementation file
//

#include "stdafx.h"
#include "SxMotion.h"
#include "SxMotionDlg.h"
#include "SpeedSetup.h"
#include "..\Common\ceSDK.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSxMotionDlg dialog

enum {JOGMODE_VEL, JOGMODE_REL, JOGMODE_ABS}TJogModeID;

#define EXPECTED_NODECOUNT	10
#define TIMEOUT	(10*100) //  ϳ 100ms

long g_nErrCode;

void CSxMotionDlg::ShowCeSdkError(int nErrCode)
{
	CString sMsg;
	sMsg.Format("ceSDK error has been occured!\nerror code = %d", nErrCode);
	MessageBox(sMsg, "cnSDK Error", MB_OK|MB_ICONERROR);
}

CSxMotionDlg::CSxMotionDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSxMotionDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSxMotionDlg)
	m_nJogMode = 0;
	m_fDistance = 10000.0;
	m_fPos1 = 0.0;
	m_fPos2 = 10000.0;
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_pStateMonDlg = NULL;
}

void CSxMotionDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSxMotionDlg)
	DDX_Control(pDX, IDC_btnMoveP, m_btnMoveP);
	DDX_Control(pDX, IDC_btnMoveN, m_btnMoveN);
	DDX_Control(pDX, IDC_SLIDER1, m_sldSpeedRatio);
	DDX_Control(pDX, IDC_cmbAxisSel, m_cmbAxisSel);
	DDX_Radio(pDX, IDC_rdoJogMode, m_nJogMode);
	DDX_Text(pDX, IDC_edtDist, m_fDistance);
	DDX_Text(pDX, IDC_edtPos1, m_fPos1);
	DDX_Text(pDX, IDC_edtPos2, m_fPos2);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CSxMotionDlg, CDialog)
	//{{AFX_MSG_MAP(CSxMotionDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_rdoJogMode, OnJogModeSelect)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_btnStop, OnStop)
	ON_BN_CLICKED(IDC_btnResetPos, OnResetPos)
	ON_BN_CLICKED(IDC_btnSpeedSetup, OnSpeedSetup)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_RADIO2, OnJogModeSelect)
	ON_BN_CLICKED(IDC_RADIO3, OnJogModeSelect)
	ON_BN_CLICKED(IDC_BTN_SHOWSTATEMON, OnBtnShowstatemon)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WMU_CHILD_LBUTTON_DOWN, OnUserLButtonDown)  // User Message Mapping
	ON_MESSAGE(WMU_CHILD_LBUTTON_UP, OnUserLButtonUp)  // User Message Mapping
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSxMotionDlg message handlers

BOOL CSxMotionDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// Load CMMSDK(DLL) Library //
	if( !ceLoadDll() ){
		MessageBox("Can't load CE-SDK Library(DLL)", "Error", MB_OK | MB_ICONERROR);
		return FALSE;
	}

	// Set Debug mode enable
	//ceGnDebugMode(2, NULL);

	// Load Motion & DIO devices //
	long nNumNodes;
	int i;

	//̽ ε մϴ.
	g_nErrCode = ceGnLoad();

	if( g_nErrCode != ceERR_NONE ) 
	{
		ShowCeSdkError(g_nErrCode);
		return FALSE;
	}		

	long nIsSearchedDev = 0;

	//̹ ˻  Ȯմϴ.
	ceGnIsSearchedDevice( &nIsSearchedDev );

	if ( nIsSearchedDev == CE_FALSE )
	{
		//带 ˻մϴ.
		if((g_nErrCode = ceGnSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, FALSE, &nNumNodes)) < 0) {
			ShowCeSdkError(g_nErrCode);
			return FALSE;
		}		
	}
	else
	{
		//带 ˻մϴ.
		if((g_nErrCode = ceGnReSearchDevice(EXPECTED_NODECOUNT, TIMEOUT, FALSE, &nNumNodes)) < 0) {
			ShowCeSdkError(g_nErrCode);
			return FALSE;
		}		
	}

	LONG nNodes = 0;

	ceGnTotalNode( &nNodes );

	ceGnTotalMotionChannel( &m_nNumAxes );

	//ConfigMotion();
	m_pStateMonDlg = new CStateMonitorDlg(this);
	VERIFY(m_pStateMonDlg->Create(IDD_STATEMONITOR_DIALOG, this));

	////////////////////////////////////////////////////////////////////////////////////
	// ༳ ޺ڽ ฮƮ äֽϴ. 
	char szAxisTitle[20];
	m_cmbAxisSel.ResetContent(); // Clear combobox
	for(i=0; i<m_nNumAxes; i++) {
		sprintf(szAxisTitle, "Axis %d", i);
		m_cmbAxisSel.AddString(szAxisTitle);
	}
	m_cmbAxisSel.SetCurSel(0);
	// Speed ratio ̴    //
	m_sldSpeedRatio.SetRange(1, 100);
	m_sldSpeedRatio.SetPos(100);

	///////////////////////////////////////////////////////////////////////
	// Assign "tag value" to each button => this is done in order to know 
	// which button generated "LBUTTON_DOWN / LBUTTON_UP" event
	m_btnMoveN.SetTag(IDC_btnMoveN);
	m_btnMoveP.SetTag(IDC_btnMoveP);

	OnJogModeSelect(); // Jog 忡   Ʈڽ Dimmed ó ؼ...

	// Set timer to display current command position //
	SetTimer(0, 100, NULL); 
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}


void CSxMotionDlg::ConfigMotion()
{
	for(int i=0; i<m_nNumAxes; i++)
	{
		cemCfgMioProperty_Set(i, cemMPID_ALM_LOGIC, cemLOGIC_A);
		cemCfgMioProperty_Set(i, cemMPID_EL_LOGIC, cemLOGIC_A);
		cemCfgInMode_Set(i, cemIMODE_AB4X, false);
		cemCfgOutMode_Set(i, cemOMODE_CWCCW0);
	}
}


void CSxMotionDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this);

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CSxMotionDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CSxMotionDlg::OnJogModeSelect() 
{
	UpdateData();
	switch(m_nJogMode)
	{
	case JOGMODE_VEL: // "Velocity-Move" Mode
		GetDlgItem(IDC_lblDist)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtDist)->EnableWindow(FALSE);
		GetDlgItem(IDC_lblPos1)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtPos1)->EnableWindow(FALSE);
		GetDlgItem(IDC_lblPos2)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtPos2)->EnableWindow(FALSE);
		GetDlgItem(IDC_btnStop)->EnableWindow(FALSE);
		GetDlgItem(IDC_btnMoveN)->SetWindowText("Jog (-)");
		GetDlgItem(IDC_btnMoveP)->SetWindowText("Jog (+)");
		break;

	case JOGMODE_REL: // "Relative Position Move" Mode
		GetDlgItem(IDC_lblDist)->EnableWindow(TRUE);
		GetDlgItem(IDC_edtDist)->EnableWindow(TRUE);
		GetDlgItem(IDC_lblPos1)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtPos1)->EnableWindow(FALSE);
		GetDlgItem(IDC_lblPos2)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtPos2)->EnableWindow(FALSE);
		GetDlgItem(IDC_btnStop)->EnableWindow(TRUE);
		GetDlgItem(IDC_btnMoveN)->SetWindowText("Move (-)");
		GetDlgItem(IDC_btnMoveP)->SetWindowText("Move (+)");
		break;

	case JOGMODE_ABS: // "Absolute Position Move" Mode
		GetDlgItem(IDC_lblDist)->EnableWindow(FALSE);
		GetDlgItem(IDC_edtDist)->EnableWindow(FALSE);
		GetDlgItem(IDC_lblPos1)->EnableWindow(TRUE);
		GetDlgItem(IDC_edtPos1)->EnableWindow(TRUE);
		GetDlgItem(IDC_lblPos2)->EnableWindow(TRUE);
		GetDlgItem(IDC_edtPos2)->EnableWindow(TRUE);
		GetDlgItem(IDC_btnStop)->EnableWindow(TRUE);
		GetDlgItem(IDC_btnMoveN)->SetWindowText("Position1");
		GetDlgItem(IDC_btnMoveP)->SetWindowText("Position2");
		break;
	}
}


/***********************************************************************
* OnTimer() : WM_TIMER event handler
* : Read current command position and display it
***********************************************************************/
void CSxMotionDlg::OnTimer(UINT nIDEvent) 
{
	UpdateData();
	
	//-------  ġ ͸ -------------------------------//
	m_nAxis = m_cmbAxisSel.GetCurSel();
	CString sPos;
	double fCurPos;
	// Command position //
	cemStPosition_Get(m_nAxis, cemCNT_COMM, &fCurPos);
	sPos.Format("%0.16G", fCurPos);
	GetDlgItem(IDC_lblPosC)->SetWindowText(sPos);
	// Feedback position //
	cemStPosition_Get(m_nAxis, cemCNT_FEED, &fCurPos);
	sPos.Format("%0.16G", fCurPos);
	GetDlgItem(IDC_lblPosF)->SetWindowText(sPos);

	//-------- Speed ratio  ȭ鿡 ڷ ǥϱ ------------//
	int nRatio = m_sldSpeedRatio.GetPos();
	CString sTemp;
	sTemp.Format("%d (%s)", nRatio, "%");
	GetDlgItem(IDC_lblSpeedRatio)->SetWindowText(sTemp);

	UpdateData(FALSE);
	
	CDialog::OnTimer(nIDEvent);
}

/***********************************************************************
* OnResetPos() : Position "Reset" ư ݹԼ. Command & Feedback 
* position 0 Ѵ.
***********************************************************************/
void CSxMotionDlg::OnResetPos() 
{
	m_nAxis = m_cmbAxisSel.GetCurSel();
	cemStPosition_Set(m_nAxis, cemCNT_COMM, 0.0); // Reset COMMAND Position
	cemStPosition_Set(m_nAxis, cemCNT_FEED, 0.0); // Reset FEEDBACK Position
}


/***********************************************************************
* OnSpeedSetup() : "Speed Setup" Button's event handler. ӵ 
*    ִ ȭڸ ÷ϰ  ӵ Ʈѷ
* մϴ.   cemCfgSpeedPattern_Set() Լ  ϱ
*   ڵԴϴ. ׷ Ϲ ӵ CME Ͽ 
* ϹǷ cemCfgSpeedPattern_Set() Լ  ʿ䰡  찡 ϴ. 
* ſ, cemSxSpeedRatio_Set() Լ ̿Ͽ ̼ۼӵ ϴ 
* ϹԴϴ.
***********************************************************************/
void CSxMotionDlg::OnSpeedSetup() 
{
	UpdateData(); 
	m_nAxis = m_cmbAxisSel.GetCurSel();
	CSpeedSetup SpeedSetup;	
	SpeedSetup.SetAxis(m_nAxis);
	// ȭ ÷ //
	if(SpeedSetup.DoModal() == IDOK){
		// ڰ  ӵ  Ѵ. //
		cemCfgSpeedPattern_Set(m_nAxis, SpeedSetup.m_nSpeedMode, SpeedSetup.m_fVwork, SpeedSetup.m_fAcc, SpeedSetup.m_fDec);
	}
}

/***********************************************************************
* OnUserLButtonDown() : ư WM_LBUTTONDOWN ̺Ʈ ڵ鷯Դϴ.
* CButton Ŭ WM_LBUTTONDOWN ̺Ʈ  Ƿ CJogButton
* ̶ Ŭ Ͽ WM_LBUTTONDOWN ̺Ʈ Ŀ User-event
* CSxMotionDlg Ŭ ϴ  Ͽϴ.
***********************************************************************/
LONG CSxMotionDlg::OnUserLButtonDown(WPARAM wParam, LPARAM lParam)
{
	UpdateData(); 

	// Velocity-Move(Jog) ϶ WM_LBUTTONDOWN ̺Ʈ óѴ. //
	// Rel/Abs Position Mode WM_LBUTTONUP ̺Ʈ ̼  //
	// óѴ.                                                       //
	if(m_nJogMode == JOGMODE_VEL){
		m_nAxis = m_cmbAxisSel.GetCurSel(); // Get current user's axis selection

		LONG nResult;
		//   .
		double fVelR = (double)m_sldSpeedRatio.GetPos();

		//long nSpdMode;
		//double fAccR, fDecR;
		//nResult = cemSxSpeedRatio_Get(m_nAxis, &nSpdMode, &fVelR, &fAccR, &fDecR);
		nResult = cemSxSpeedRatio_Set(m_nAxis, cemSMODE_KEEP/*Keep prev. setting*/, fVelR, 100, 100); // Speed ratio :  ǻ 100% 

		CJogButton *pButton = (CJogButton *)lParam;
		if(pButton->GetTag() == IDC_btnMoveN){ // "Jog (-)" ư  ...
			if((g_nErrCode = cemSxVMoveStart(m_nAxis, cemDIR_N)) != ceERR_NONE){ 
				//ShowCnSdkError(g_nErrCode);
			}
		}else{ // "Jog (+)" ư  ...
			if((g_nErrCode = cemSxVMoveStart(m_nAxis, cemDIR_P)) != ceERR_NONE){ 
				//ShowCnSdkError(g_nErrCode);
			}
		}
	}
	
	return 0;
}
/***********************************************************************
* OnUserLButtonUp() : ư WM_LBUTTONUP ̺Ʈ ڵ鷯Դϴ. ⼭
* JOG ̼ մϴ.
***********************************************************************/
LONG CSxMotionDlg::OnUserLButtonUp(WPARAM wParam, LPARAM lParam)
{
	m_nAxis = m_cmbAxisSel.GetCurSel(); // Get current user's axis selection
	if(m_nJogMode == JOGMODE_VEL){ // Velocity-Move(Jog) 忡 WM_LBUTTONUP ̺Ʈÿ ͸ Ѵ.
		LONG nResult = cemSxStop(m_nAxis, FALSE, TRUE);
		if(nResult < ceERR_NONE)
		{
			int i = 0;
		}
	}else{ //  忡 WM_LBUTTONUP ̺Ʈ ̼ Ѵ.
		// ̼  ߺϴ  ʵ ̼۹ư Disable //
		GetDlgItem(IDC_btnMoveN)->EnableWindow(FALSE);
		GetDlgItem(IDC_btnMoveP)->EnableWindow(FALSE);

		CJogButton *pButton = (CJogButton *)lParam;

		//   .
		double fVelR = (double)m_sldSpeedRatio.GetPos();
		LONG nResult = cemSxSpeedRatio_Set(m_nAxis, cemSMODE_KEEP/*Keep prev. setting*/, fVelR, 100, 100); // Speed ratio :  ǻ 100% 

		if(m_nJogMode == JOGMODE_REL){ // ǥ ̼ ϶
			// "Move (-)"ư̸ Distance  ϰ, "Move (+)" ư̸  Ѵ //
			double fDistance = (pButton->GetTag() == IDC_btnMoveN) ? -m_fDistance: m_fDistance;
			///////////////////////////////////////////////////////////////////////////////////
			// [CAUTION] cmmSxMove() Լ IsBlocking ĶͿ Ͽ:
			//  ǻ ̼߿ Ÿ̸ ̺Ʈ Stop ư ̺Ʈ ó   ֵ 
			// IsBlocking Ķ͸ FALSE Ͽ.  带   IsBlocking 
			// Ķ͸ ݵ TRUE ؾѴ. 
			//Sleep(1000);
			if((g_nErrCode = cemSxMoveStart(m_nAxis, fDistance)) != ceERR_NONE){ 
				//ShowCnSdkError(g_nErrCode);
			}
			cemSxWaitDone(m_nAxis, false);
		}else{
			double fPosition = (pButton->GetTag() == IDC_btnMoveN) ? m_fPos1: m_fPos2;
			///////////////////////////////////////////////////////////////////////////////////
			// [CAUTION] cmmSxMove() Լ IsBlocking ĶͿ Ͽ:
			//  ǻ ̼߿ Ÿ̸ ̺Ʈ Stop ư ̺Ʈ ó   ֵ 
			// IsBlocking Ķ͸ FALSE Ͽ.  带   IsBlocking 
			// Ķ͸ ݵ TRUE ؾѴ. 
			if((g_nErrCode = cemSxMoveToStart(m_nAxis, fPosition)) != ceERR_NONE){ 
				//ShowCnSdkError(g_nErrCode);
			}
			cemSxWaitDone(m_nAxis, false);
		}

		GetDlgItem(IDC_btnMoveN)->EnableWindow(TRUE);
		GetDlgItem(IDC_btnMoveP)->EnableWindow(TRUE);
	}

	
	return 0;
}


/***********************************************************************
* OnStop() : "Stop" Button's event handler
* : This button stops motion of specifed axis
***********************************************************************/
void CSxMotionDlg::OnStop() 
{
	m_nAxis = m_cmbAxisSel.GetCurSel();
	GetDlgItem(IDC_btnStop)->EnableWindow(FALSE);
	if((g_nErrCode = cemSxStop(m_nAxis, TRUE, FALSE)) != ceERR_NONE){
		//ShowCnSdkError(g_nErrCode);
	}
	GetDlgItem(IDC_btnStop)->EnableWindow(TRUE);
}

void CSxMotionDlg::OnDestroy() 
{
	KillTimer(0);

	if(m_pStateMonDlg != NULL)
	{
		delete m_pStateMonDlg;
		m_pStateMonDlg = NULL;
	}

	m_nAxis = m_cmbAxisSel.GetCurSel();

	//	cemSxStop(m_nAxis, FALSE, FALSE);

	ceGnUnload(); // Unload Device
	
	ceUnloadDll(); // Unload DLL

	CDialog::OnDestroy();
}

void CSxMotionDlg::OnBtnShowstatemon() 
{
	if(m_pStateMonDlg != NULL)
	{
		m_pStateMonDlg->ShowWindow(SW_SHOW);
	}
}
